/****************************************************************************
** DVDMenuUndoObject - code
**
**   Created : Thursday Jan 2'nd 2003
**        by : Varol Okan
** Copyright : (c) Varol Okan
**   License : GPL v 2.0
**
****************************************************************************/

#include <qpixmap.h>

#include "global.h"
#include "dvdmenu.h"
#include "qdvdauthor.h"
#include "menuobject.h"
#include "menupreview.h"
#include "sourcefileentry.h"
#include "dvdmenuundoobject.h"

DVDMenuUndoObject::DVDMenuUndoObject ( )
{
  initMe ( );
}

DVDMenuUndoObject::DVDMenuUndoObject ( enUndoType type, MenuObject *pMenuObject, ButtonPreview *pButtonPreview, int iTwiggle )
{
  initMe ( );
  m_iTwiggle       = iTwiggle; // Used for moving with cursor keys.
  m_enUndoType     = (int)type;
  m_pMenuObject    = pMenuObject;
  m_pButtonPreview = pButtonPreview;
  switch ( m_enUndoType )  {
  case MOVED_OBJECT :
    m_rectPos = pMenuObject->rect ( );
  break;
  case DELETED_OBJECT :
    m_bDeleteObject = true;
    m_pMenuObject->resetStructureItem ( );
  break;
  }
}

DVDMenuUndoObject::DVDMenuUndoObject ( enUndoType type, MenuObject *pMenuObject, MenuObject *pShadow )
{
  initMe ( );
  m_enUndoType     = (int)type;
  m_pMenuObject    = pMenuObject;
  m_pShadowObject  = pShadow ? pShadow->clone ( pMenuObject->parent ( ), pMenuObject ) : NULL;
  m_bDeleteObject  = true;
  if ( ( m_enUndoType == EDIT_OBJECT     ) ||
       ( m_enUndoType == DELETED_SHADOW  ) ||
       ( m_enUndoType == DELETED_OVERLAY ) )  {
    m_bDeleteObject = true;
    m_pMenuObject->resetStructureItem ( );
  }
}

DVDMenuUndoObject::DVDMenuUndoObject ( MenuObject *pMenuObject )
{
  initMe ( );
  m_enUndoType     = (int)EDIT_OBJECT;
  m_pMenuObject    = pMenuObject;
  m_pObjectData    = pMenuObject->data ( )->clone ( );
  m_bDeleteObject  = true;
}

DVDMenuUndoObject::DVDMenuUndoObject ( enUndoType type, QPixmap &theOldBackground, CDVDMenuInterface *pInterface )
{
  initMe ( );
  if ( type != NEW_BACKGROUND )
    return;

  m_enUndoType = (int)type;
  m_backgroundInfo.background = theOldBackground;
  m_backgroundInfo.getBackgroundInfo ( pInterface );
}

DVDMenuUndoObject::DVDMenuUndoObject ( enUndoType type, SourceFileEntry *pSourceFileEntry )
{
  initMe ( );
  if ( type != NEW_AUDIO )
    return;
  m_enUndoType       = (int)type;
  m_pSourceFileEntry = pSourceFileEntry;
}

DVDMenuUndoObject::~DVDMenuUndoObject ( )
{
  // Okay if we have a m_pMenuObject and this UndoObject is to be deleted 
  // then we want to delete the m_pMenuObject only if this UndoObject is deleted while 
  // the UndoBuffer::redoBuffer is emptied 
  // BUT NOT when the UndoObject is deleted due to the MAX_UNDO_BUFFER is reached.
//printf ( "%s::%s::%d > \n", __FILE__, __FUNCTION__, __LINE__ );

  if ( ( m_enUndoType == NEW_SHADOW  ) || ( m_enUndoType == DELETED_SHADOW  ) ||
       ( m_enUndoType == NEW_OVERLAY ) || ( m_enUndoType == DELETED_OVERLAY ) )  {
    if ( m_bDeleteObject && m_pShadowObject )
      delete m_pShadowObject;
  }
  else  {
    if ( m_bDeleteObject && m_pObjectData )
      delete m_pObjectData;
    else if ( m_bDeleteObject && m_pMenuObject )
      delete m_pMenuObject;
  }
  if ( m_bDeleteObject && m_pSourceFileEntry )
    delete m_pSourceFileEntry;
}

void DVDMenuUndoObject::changeType ( enUndoType type )
{
  m_enUndoType = type;
}

void DVDMenuUndoObject::initMe ( )
{
	m_enUndoType        = (int)UNDEFINED;
	m_iTwiggle          =   -1;
	m_pMenuObject       = NULL;
	m_pSourceFileEntry  = NULL;
	m_pButtonPreview    = NULL;
	m_pShadowObject     = NULL;
	m_pObjectData       = NULL;
	m_bDeleteObject     = false;
}

DVDMenuUndoObject::BackgroundInfo::BackgroundInfo ( )
{
	bModified = false;
	bMovieBackgroundHasSound = false;
	iFormat = 0;
	iHeight = 0;
	iWidth  = 0;
	iStretchType = -1;
}

QString DVDMenuUndoObject::getTypeString ( )
{
  QString qsType;
  switch ( m_enUndoType )  {
  case DRAGGED_TEXT :
    qsType = QObject::tr ( "New Text" );
  break;
  case DRAGGED_IMAGE :
    qsType = QObject::tr ( "New Image" );
  break;
  case DRAGGED_MOVIE :
    qsType = QObject::tr ( "New Clip" );
  break;
  case DRAGGED_BUTTON :
    qsType = QObject::tr ( "New Button" );
  break;
  case MOVED_OBJECT :
    qsType = QObject::tr ( "Move" );
  break;
  case NEW_BACKGROUND :
    qsType = QObject::tr ( "New Background" );
  break;
  case NEW_AUDIO :
    qsType = QObject::tr ( "New Sound" );
  break;
  case NEW_OBJECT :
    qsType = QObject::tr ( "New" );
  break;
  case EDIT_OBJECT :
    qsType = QObject::tr ( "Edited" );
  break;
  case DELETED_OBJECT :
    qsType = QObject::tr ( "Deleted" );
  break;
  case NEW_SHADOW :
    qsType = QObject::tr ( "New Shadow" );
  break;
  case DELETED_SHADOW :
    qsType = QObject::tr ( "Del Shadow" );
  break;
  case NEW_OVERLAY :
    qsType = QObject::tr ( "New Mask" );
  break;
  case DELETED_OVERLAY :
    qsType = QObject::tr ( "Del Mask" );
  break;
  default:
    qsType = QObject::tr ( "Undefined" );
  }
  return qsType;
}

// Here is the code, to restore the previous configuration.
void DVDMenuUndoObject::undo ( DVDMenu *pDVDMenu )
{
  QRect rectPos;
  QPixmap tempPix;
  BackgroundInfo tempInfo;
  CDVDMenuInterface *pInterface = NULL;

  switch ( m_enUndoType )  {
  case DRAGGED_IMAGE  :
  case DRAGGED_MOVIE  :
  case DRAGGED_TEXT   :
  case DRAGGED_BUTTON :
  case NEW_OBJECT     :
    pDVDMenu->getMenuPreview ( )->removeObject ( m_pMenuObject );
    m_bDeleteObject = true;
  break;
  case MOVED_OBJECT   :
    rectPos = m_pMenuObject->rect ( );
    m_pMenuObject->setRect ( m_rectPos );
    m_rectPos = rectPos;
  break;
  case NEW_BACKGROUND :
    // First we change the Pixmap
    tempPix = *pDVDMenu->getMenuPreview()->paletteBackgroundPixmap ();
    pDVDMenu->getMenuPreview()->setPixmap (m_backgroundInfo.background);
    // STore the prvious Pixmap
    m_backgroundInfo.background = tempPix;
    // Next we get additional information
    pInterface = pDVDMenu->getInterface();
    tempInfo.getBackgroundInfo(pInterface);
    m_backgroundInfo.setBackgroundInfo(pInterface);
    m_backgroundInfo = tempInfo;
    pDVDMenu->getMenuPreview ( )->resize       ( 720, 480 );
    pDVDMenu->getMenuPreview ( )->setFixedSize ( 720, 480 );
    pDVDMenu->updateDVDMenu  ( );
  break;
  case NEW_AUDIO :
    // We remove the SourceFileEntry from the list and from the SourceToolBar
    pDVDMenu->removeSourceFileEntry (m_pSourceFileEntry);
    // last we mark the entry for deletion if this undo-object gets deleted.
    m_bDeleteObject = true;
  break;
  case EDIT_OBJECT  :
    handleObject ( );
  break;
  case DELETED_OBJECT :
    m_pButtonPreview->addMenuObject ( m_pMenuObject );
    // last we make sure the entry is preserved when the UndoObject is deleted
    m_bDeleteObject = false;
  break;
  case NEW_SHADOW :
  case DELETED_SHADOW :
    handleShadow ( );
  break;
  case NEW_OVERLAY :
  case DELETED_OVERLAY :
    handleOverlay ( );
  break;
  };  // End of the switch loop.
  pDVDMenu->refresh ( );
}

void DVDMenuUndoObject::redo ( DVDMenu *pDVDMenu )
{
  QRect rectPos;
  QPixmap tempPix;
  BackgroundInfo tempInfo;
  CDVDMenuInterface *pInterface = NULL;

  switch ( m_enUndoType )  {
  case DRAGGED_IMAGE  :
  case DRAGGED_MOVIE  :
  case DRAGGED_TEXT   :
  case DRAGGED_BUTTON :
  case NEW_OBJECT     :
    // inserted dragged text, image or button ... All's the smae !
    pDVDMenu->getMenuPreview ( )->addMenuObject ( m_pMenuObject );
    m_bDeleteObject = false;
  break;
  case MOVED_OBJECT   :
    rectPos   = m_pMenuObject->rect  ( );
    m_pMenuObject->setRect ( m_rectPos );
    m_rectPos = rectPos;
  break;
  case NEW_BACKGROUND :
    // First we change the Pixmap
    tempPix = *pDVDMenu->getMenuPreview ( )->paletteBackgroundPixmap  ( );
    pDVDMenu->getMenuPreview ( )->setPixmap ( m_backgroundInfo.background );
    // STore the prvious Pixmap
    m_backgroundInfo.background = tempPix;
    // Next we get additional information
    pInterface = pDVDMenu->getInterface ( );
    tempInfo.getBackgroundInfo ( pInterface );
    m_backgroundInfo.setBackgroundInfo ( pInterface );
    m_backgroundInfo = tempInfo;
    pDVDMenu->getMenuPreview ( )->resize       ( 720, 480 );
    pDVDMenu->getMenuPreview ( )->setFixedSize ( 720, 480 );
    pDVDMenu->updateDVDMenu  ( );
  break;
  case NEW_AUDIO :
    pDVDMenu->appendSourceFileEntry ( m_pSourceFileEntry );
    // last we make sure the entry is preserved when the UndoObject is deleted
    m_bDeleteObject = false;
  break;
  case EDIT_OBJECT :
    handleObject ( );
  break;
  case DELETED_OBJECT :
    m_pButtonPreview->removeObject ( m_pMenuObject );
    m_pMenuObject->resetStructureItem ( );
    // last we make sure the entry is preserved when the UndoObject is deleted
    m_bDeleteObject = true;
  break;
  case NEW_SHADOW :
  case DELETED_SHADOW :
    handleShadow ( );
  break;
  case NEW_OVERLAY :
  case DELETED_OVERLAY :
    handleOverlay ( );
  break;
  };  // End of the switch loop.
  pDVDMenu->refresh ( );
}

void DVDMenuUndoObject::handleObject ( )
{
  if ( m_pMenuObject && m_pObjectData )  {
    MenuObject::Data *pObjectData = m_pMenuObject->data ( );
    if ( pObjectData )  {
      MenuObject::Data *pObject = pObjectData->clone ( );
      m_pMenuObject->setData ( m_pObjectData );
      delete m_pObjectData;
      m_pObjectData = pObject;
    }
  }
}

void DVDMenuUndoObject::handleShadow ( )
{
  MenuObject *pObject = m_pMenuObject->shadow ( );
  if ( ( m_pShadowObject ) && ( pObject ) )  {
    pObject->resetStructureItem ( );
    MenuObject *pTemp = pObject->clone ( m_pMenuObject->parent ( ), m_pMenuObject );
    m_pMenuObject->setShadow ( m_pShadowObject );
    m_pMenuObject->updateShadow ( );
    m_pShadowObject = pTemp;
  }
  else if ( ( ! m_pShadowObject ) && ( pObject ) )  {
    pObject->resetStructureItem ( );
    m_pShadowObject = pObject->clone ( m_pMenuObject->parent ( ), m_pMenuObject );
    m_pMenuObject->setShadow ( NULL );
  }
  else if ( ( m_pShadowObject ) && ( ! pObject ) )  {
    m_pMenuObject->setShadow ( m_pShadowObject );
    m_pMenuObject->updateShadow ( );
    m_pShadowObject = NULL;
  }
  Global::pApp->slotUpdateStructure ( );
}

void DVDMenuUndoObject::handleOverlay ( )
{
  MenuObject *pObject = m_pMenuObject->overlay ( );
  if ( ( m_pShadowObject ) && ( pObject ) )  {
    pObject->resetStructureItem ( );
    MenuObject *pTemp = pObject->clone ( m_pMenuObject->parent ( ), m_pMenuObject );
    m_pMenuObject->setOverlay ( m_pShadowObject );
    m_pMenuObject->updateOverlay ( );
    m_pShadowObject = pTemp;
  }
  else if ( ( ! m_pShadowObject ) && ( pObject ) )  {
    pObject->resetStructureItem ( );
    m_pShadowObject = pObject->clone ( m_pMenuObject->parent ( ), m_pMenuObject );
    m_pMenuObject->setOverlay ( NULL );
  }
  else if ( ( m_pShadowObject ) &&  ( ! pObject ) )  {
    m_pMenuObject->setOverlay ( m_pShadowObject );
    m_pMenuObject->updateOverlay ( );
    m_pShadowObject = NULL;
  }
  Global::pApp->slotUpdateStructure ( );
}

MenuObject *DVDMenuUndoObject::getMenuObject ( )
{
  return m_pMenuObject;
}

// This function is called after a MouseRelease event in the ButtonPreview class
// It'll check if the object was moved or not.
bool DVDMenuUndoObject::hasMoved ( )
{
  if ( ! m_pMenuObject )
    return false;
  if ( ( m_pMenuObject->rect ( ).x ( ) != m_rectPos.x ( ) ) ||
       ( m_pMenuObject->rect ( ).y ( ) != m_rectPos.y ( ) ) )
    return true;
  return false;
}

bool DVDMenuUndoObject::isMoving ( int iButton )
{
  // Used for determining Cursor key moving
  if ( ( m_enUndoType == MOVED_OBJECT ) && ( iButton == m_iTwiggle ) )
    return true;
  return false;
}

bool DVDMenuUndoObject::hasChanged ( )
{
  if ( m_pMenuObject && m_pObjectData && ( getType ( ) == EDIT_OBJECT ) )
    return  ( ! ( *m_pMenuObject->data ( ) == *m_pObjectData ) );

  return false;
}

DVDMenuUndoObject::BackgroundInfo &DVDMenuUndoObject::BackgroundInfo::operator = ( DVDMenuUndoObject::BackgroundInfo &theOther )
{
  bModified                = theOther.bModified;
  // the following entries belong to the movie background
  qsMovieFileName          = theOther.qsMovieFileName;
  bMovieBackgroundHasSound = theOther.bMovieBackgroundHasSound;
  timeOffset               = theOther.timeOffset;
  timeDuration             = theOther.timeDuration;
  // The following var is the background image or screenshot of themovie ...
  qsBackgroundFileName     = theOther.qsBackgroundFileName;
  qsResizeAlgo             = theOther.qsResizeAlgo;
  iStretchType             = theOther.iStretchType;	// StretchToFit, BlackBorders, CutOffset
  posOffset                = theOther.posOffset;
  iFormat                  = theOther.iFormat;
  iWidth                   = theOther.iWidth;
  iHeight                  = theOther.iHeight;
  return *this;
}

void DVDMenuUndoObject::BackgroundInfo::getBackgroundInfo ( CDVDMenuInterface *pInterface )
{
  qsBackgroundFileName     = pInterface->qsBackgroundFileName;
  bModified                = pInterface->bModified;
  // the following entries belong to the movie background
  qsMovieFileName          = pInterface->qsMovieFileName;
  bMovieBackgroundHasSound = pInterface->bMovieBackgroundHasSound;
  timeOffset               = pInterface->timeOffset;
  timeDuration             = pInterface->timeDuration;
  // The following var is the background image or screenshot of themovie ...
  qsBackgroundFileName     = pInterface->qsBackgroundFileName;
  qsResizeAlgo             = pInterface->qsResizeAlgo;
  iStretchType             = pInterface->iStretchType;	// StretchToFit, BlackBorders, CutOffset
  posOffset                = pInterface->posOffset;
  iFormat                  = pInterface->iFormat;
  iWidth                   = pInterface->iWidth;
  iHeight                  = pInterface->iHeight;
}

void DVDMenuUndoObject::BackgroundInfo::setBackgroundInfo (CDVDMenuInterface *pInterface)
{
  pInterface->qsBackgroundFileName     = qsBackgroundFileName;
  pInterface->bModified                = bModified;
  // the following entries belong to the movie background
  pInterface->qsMovieFileName          = qsMovieFileName;
  pInterface->bMovieBackgroundHasSound = bMovieBackgroundHasSound;
  pInterface->timeOffset               = timeOffset;
  pInterface->timeDuration             = timeDuration;
  // The following var is the background image or screenshot of themovie ...
  pInterface->qsBackgroundFileName     = qsBackgroundFileName;
  pInterface->qsResizeAlgo             = qsResizeAlgo;
  pInterface->iStretchType             = iStretchType;	// StretchToFit, BlackBorders, CutOffset
  pInterface->posOffset                = posOffset;
  pInterface->iFormat                  = iFormat;
  pInterface->iWidth                   = iWidth;
  pInterface->iHeight                  = iHeight;
}

