/****************************************************************************
** UndoBuffer and UndoObject classes
**   Created : Saturday Nov 30'st 2004
**        by : Varol Okan
** Copyright : (c) Varol Okan
**   License : GPL v 2.0
****************************************************************************/
#ifndef DVDMENUUNDOOBJECT_H
#define DVDMENUUNDOOBJECT_H

#include <qdatetime.h>

#include "menuobject.h"
#include "undobuffer.h"

//class MenuObject;
class QPixmap;

class CDVDMenuInterface;
class SourceFileEntry;
class ButtonPreview;
// This class will encapsulate the Undo / redo Objects for the ConstructionArea - class
// Any UndoObject will hold only the most neccesary information for undo/redo functionality 
// to work, thus no complete FunctionPainter objects or other objects will be stored herein.
///////////////////////////////////////////////////////////////////////////////////////////

class DVDMenuUndoObject : public UndoObject
{
	class BackgroundInfo	{
	public:
		BackgroundInfo ( );
		void getBackgroundInfo (CDVDMenuInterface  *);
		void setBackgroundInfo (CDVDMenuInterface  *);
		BackgroundInfo &operator = (BackgroundInfo &);

		QPixmap background;
		bool bModified;

		// the following entries belong to the movie background
		QString qsMovieFileName;
		bool    bMovieBackgroundHasSound;
		QTime   timeOffset;
		QTime   timeDuration;
		// The following var is the background image or screenshot of themovie ...
		QString qsBackgroundFileName;

		QString qsResizeAlgo;
		int iStretchType;	// StretchToFit, BlackBorders, CutOffset
		QSize posOffset;
		int iFormat;
		int iWidth;
		int iHeight;
	};

public:
	enum enUndoType { UNDEFINED = 0, DRAGGED_TEXT, DRAGGED_IMAGE, DRAGGED_MOVIE, DRAGGED_BUTTON, MOVED_OBJECT, NEW_BACKGROUND, NEW_AUDIO, NEW_OBJECT, EDIT_OBJECT, DELETED_OBJECT, NEW_SHADOW, DELETED_SHADOW, NEW_OVERLAY, DELETED_OVERLAY };

	DVDMenuUndoObject ( enUndoType, MenuObject *, ButtonPreview  *p=NULL, int i=-1 );
	DVDMenuUndoObject ( enUndoType, MenuObject *, MenuObject     * );
	DVDMenuUndoObject ( enUndoType, QPixmap &, CDVDMenuInterface * );
	DVDMenuUndoObject ( enUndoType, SourceFileEntry * );
	DVDMenuUndoObject ( MenuObject * );
	virtual ~DVDMenuUndoObject();

	virtual bool hasChanged       ( );
	virtual void undo   ( DVDMenu * );
	virtual void redo   ( DVDMenu * );

	virtual QString getTypeString ( );
	MenuObject *getMenuObject     ( );
	void    changeType ( enUndoType );
	bool        isMoving      ( int );
	bool        hasMoved          ( );

private:
	DVDMenuUndoObject  ( );
	void initMe        ( );
	void handleObject  ( );
	void handleShadow  ( );
	void handleOverlay ( );

private:
	// Private member variables ...
	MenuObject       *m_pMenuObject;
	MenuObject       *m_pShadowObject; // ShadowObject OR OverlayObject
	MenuObject::Data *m_pObjectData;
	bool              m_bDeleteObject;
	int               m_iTwiggle; // Always wanted to call a variable twiggle
	QRect             m_rectPos;
	BackgroundInfo    m_backgroundInfo;
	SourceFileEntry  *m_pSourceFileEntry;
	ButtonPreview    *m_pButtonPreview;
};

#endif // DVDMENUUNDOOBJECT_H

